# Object that contains the referenceNum, ledgerCode, journalCode, transDate, payeePayer, memo, remember
# and an array of JournalEntryRows.
#$Id: HTMLJournalEntry.pm,v 1.3 2004/02/11 22:02:30 moreejt Exp $
#This package is released under the GNU General Public License.
#You should have recieved a copy of the GPL with it.
# Copyright (c) 2000 HLR http://www.pcxperience.org  All rights reserved.

package Portal::Accounting::Objects::HTMLJournalEntry;

use strict;
use Portal::Accounting::Objects::JournalEntry;
use vars qw($VERSION @ISA @EXPORT $AUTOLOAD);

require Exporter;

@ISA = qw(Portal::Accounting::Objects::JournalEntry Exporter AutoLoader);
@EXPORT = qw();

$VERSION = '$Revision: 1.3 $';

sub new
{
  my $class = shift;
  my $self = $class->SUPER::new(@_);

  return $self;
}

=item string printXML(fullXML bool, indent int)

 This routine returns an XML snippet for this object for use in such
 things as exporting data

 fullXML - defaults to 0 (false).  If true, we output the xml header tag.
 indent - defaults to 2.  Specifies how many spaces in our top level tag
          needs to be.  If fullXML is true, then we force 0, otherwise we
          use what the user specifies.

=cut

sub printXML
{
  my $self = shift;
  my $result = "";
  my %args = ( fullXML => 0, indent => 2, @_ );
  my $fullXML = $args{fullXML};
  my $indent = ($fullXML ? 0 : $args{indent});  # start at 0 indentation if we are making a full XML document.
  my $spaceStr = " " x $indent;

  my %mapping = ("id" => "referenceNum", "remembered" => "remember", "ledgerCode" => "ledgerCode",
                 "memo" => "memo", "journalCode" => "journalCode", "date" => "transDate",
                 "payeePayer" => "payeePayer", "origId" => "origId");

  if ($self->isValid())
  {
    if ($fullXML)
    {
      $result .= "<?xml version=\"1.0\" encoding=\"ISO-8859-1\"?>";
    }
    $result .= $spaceStr . "<journalEntry>\n";
    my @tags = qw{id remembered memo journalCode date payeePayer origId};
    push @tags, "ledgerCode" if ($fullXML);
    foreach my $name (@tags)
    {
      $result .= $spaceStr . "  <$name>" . $self->{$mapping{$name}} . "</$name>\n";
    }
    # now output the rowObjects
    $result .= $spaceStr . "  <lines>\n";
    foreach my $row (@{$self->{rowObjects}})
    {
      $result .= $row->printXML(fullXML => 0, indent => $indent + 4);
    }
    $result .= $spaceStr . "  </lines>\n";
    $result .= $spaceStr . "</journalEntry>\n";
  }
  else
  {
    $result .= "Data not valid!\n\n";
    $result .= $self->errorMessage();
    die $result;
  }

  return $result;
}

=item scalar populate

=cut
sub populate
{
  my $self = shift;
  my %args = ( input => undef, @_ );

  if (defined $args{input})
  {
    $self->{referenceNum} = $args{input}->{referenceNum};
    $self->{ledgerCode}   = $args{input}->{'ledgerCode'};
    $self->{journalCode}  = $args{input}->{'journalCode'};
    $self->{transDate}    = $args{input}->{'transDate'};
    $self->{payeePayer}   = $args{input}->{'payeePayer'};
    $self->{memo}         = $args{input}->{'memo'};
    $self->{remember}     = $args{input}->{'remember'};
    $self->{trackingId}   = $args{input}->{trackingId};
    $self->{origId}       = $args{input}->{origId};
    #$self->{rowObjects}   = $args{input}->{'rowObjects'};
    if (defined $args{input}->{rowObjects})
    {
      $self->{rowObjects}   = $args{input}->{'rowObjects'};
    }
  }
  else
  {
    return $self->SUPER::poplulate(%args);
  }

  return $self->isValid;
}

=item string printForm(name => , empty => y/n, )

 This routine returns an HTML form for this object for use on a page.
 The user is supposed to fill in the form if empty = y.
 Otherwise the form will validate each element and act appropriately
 The input hash is given so that the form can pull the data from the input stream.

=cut

sub printForm
{
  my $self = shift;
  my %args = ( name => "mainForm", empty => "n", input => undef, command => "", @_ );
  if (defined $args{input})
  {  $self->{input} = $args{input}; }
  else
  {
    $self->missing(name => "input");
    return undef;
  }
  my $command;
  if ($args{command} eq "")
  {
    $command = $self->{input}->{command} ;
  }
  else
  {
    $command = $args{command} ;
  }
  my $empty = $args{empty};
  my $flag = "*";
  my %flags;
  my $name = $args{name};
  my $result = "";

  if ($empty !~ /^y|n$/)
  {
    $self->invalid(name => "empty", value => $empty);
    $self->prefixError($self->genErrorString("invalid"));
    return undef;
  }
  if ($name eq "")
  {
    $self->missing(name => "name");
    $self->prefixError($self->genErrorString("missing"));
    return undef;
  }
  if ($empty eq "n")
  {  #validate this data
    foreach my $element ("journalCode", "transDate", "payeePayer", "memo", "remember") #"trackingId" "origId"
    {
      if ($self->isEntryInvalid($element))
      {
        $flags{$element} = $flag;
      }
    }
  }
  $result .= <<"EOF";
  <form name="$name" method="post" action="$self->{baseURL}">
  <input type="hidden" name="app" value="$self->{input}->{app}">
  <input type="hidden" name="state" value="$self->{input}->{state}">
  <input type="hidden" name="command" value="$command">
  <input type="hidden" name="ledger" value="$self->{input}->{ledger}">
  <input type="hidden" name="ledgerCode" value="$self->{input}->{ledgerCode}">
  <input type="hidden" name="referenceNum" value="$self->{input}->{referenceNum}">
  $self->{referenceNum}
  <br>Date<input type="text" name="transDate" value="$self->{transDate}"> $flags{transDate}
  <br>Payee<input type="text" name="payeePayer" value="$self->{payeePayer}"> $flags{payeePayer}
  <br>J<input type="text" name="journalCode" value="$self->{journalCode}"> $flags{journalCode}
  <br>Memo<input type="text" name="memo" value="$self->{memo}"> $flags{memo}
  <br>R<input type="text" name="remember" value="$self->{remember}"> $flags{remember}
  <br><input type="submit">
EOF
  $result .= <<"EOF";
  </form>
EOF
  return $result;
}

1;
__END__

=head1 NAME

JournalEntry - Object used to build a complete JournalEntry for storage in the accounting database.

=head1 SYNOPSIS

  use JournalEntry;
  my $transobj = JournalEntry->new(referenceNum => '379',
  					ledgerCode => 'ts1',
  					journalCode => 'G',
  					transDate => '12-14-1999',
  					payeePayer => 'Bank',
  					memo => 'cashed check',
  					remember => 'false',
  					rowObjects => \@RowObjectArray,
            origId => "");
  if ($transobj->didErrorOccur())
  {
    die $transobj->errorMessage();
  }

  # set object data by specifying the data field desired
  $transobj->journalCode("I");
  $transobj->memo("");

  # make sure the object is useful by calling the validation routine
  # before writing it to the database
  if ($transobj->isValid())
  {
    updateJournalEntry($transobj);
  }
  else
  {
    die $transobj->errorMessage();
  }

=head1 DESCRIPTION

JournalEntry is a class that stores data for an accounting
JournalEntry.  It provides a container for a JournalEntry that will be used
to write to the database or update an existing record in the database.

=head1 Exported FUNCTIONS

  scalar new(referenceNum, ledgerCode, journalCode, transDate,
             payeePayer, memo, remember, rowObjects, trackingId, origId)
    Creates a new instance of the Portal::Accounting::Objects::JournalEntry
    object.  A new instance can be created in an invalid state (i.e., missing
    fields, etc.).  isValid() must be called on the object before using the
    object in a database JournalEntry.

    trackingId = the genSysId value from an XML JournalEntry Import. Can be "".
    origId = the original referenceNum value for this entry.  Can be "".

  void setError(errorString)
    Sets error = 1 and $self->{errorString} = errorString.

  scalar didErrorOccur(void)
    Returns the value of error.

  scalar errorMessage(void)
    Returns the value of errorString.

  scalar isValid(void)
    Runs through the data fields contained in the class to make sure
    they contain valid data.  This function should be called before
    using this object in a database JournalEntry.

  scalar addRow(rowDate, comment, subEntity, accountNum, subAccountNum,
                dorc, amount, cleared)
    Adds a row to the rowObjects array in the JournalEntry.
    Returns the row number of the newly added row or 0 on failure.

  scalar deleteRow(row)
    Removes the row specified from the rowObjects array.

  NOTE:  All data fields are accessible by specifying the object
         and pointing to the data member to be modified on the
         left-hand side of the assignment.

=head1 AUTHOR

James A. Pattie (mailto:james@pcxperience.com), Jason A. Pattie (mailto:pattieja@pcxperience.com)

=head1 SEE ALSO

perl(1), JournalEntry(3), DBIWrapper(3)

=cut
