# LotNodeObject.pm - The Object Class that provides a LotNode Object
# Created by James Pattie & Jason A. Pattie, 11/07/2000.
#$Id: LotNodeObject.pm,v 1.1.1.1 2001/08/09 19:04:49 moreejt Exp $
#This package is released under the GNU General Public License.
#You should have recieved a copy of the GPL with it.
# Copyright (c) 2000 HLR http://www.pcxperience.org  All rights reserved.

package Portal::Accounting::Objects::LotNodeObject;
use strict;
use vars qw($AUTOLOAD $VERSION @ISA @EXPORT);

require Exporter;

@ISA = qw(Exporter AutoLoader);
@EXPORT = qw();
	
$VERSION = '0.01';

# new 
# instantiates an instance of a LotNodeObject
# takes: ledgerCode, eventDate, parentId, eventRef, journalRef, accountNum, subAccountNum, lot, buyLot, shares, cost, nodeId
sub new
{
  my $that = shift;
  my $class = ref($that) || $that;
  my $self = bless {}, $class;
  my %args = ( ledgerCode => "", eventDate => "", parentId => -1, eventRef => -1, journalRef => -1, subEntity => "",
               accountNum => 0, subAccountNum => 0, lot => 0, buyLot => 0, shares => "0.0", cost => "0.00",
               nodeId => -1, @_ );

  $self->{ledgerCode} = $args{ledgerCode};
  $self->{eventDate} = $args{eventDate};
  $self->{parentId} = $args{parentId};
  $self->{eventRef} = $args{eventRef};
  $self->{journalRef} = $args{journalRef};
  $self->{subEntity} = $args{subEntity};
  $self->{accountNum} = $args{accountNum};
  $self->{subAccountNum} = $args{subAccountNum};
  $self->{lot} = $args{lot};
  $self->{buyLot} = $args{buyLot};
  $self->{shares} = $args{shares};
  $self->{cost} = $args{cost};
  $self->{nodeId} = $args{nodeId};

  # do validation
  $self->{error} = !$self->isValid;  
  if ($self->{error})
  {
    $self->{errorString} = "LotNodeObject: new() - Error!<br>\n" . $self->{errorString};
  }

  return $self;
}

# isValid - Returns 0 or 1 to indicate if the object is valid.
sub isValid
{
  my $self = shift;
  my $error = 0;
  my $errorString = "";

  # do validation code here.
  if (length $self->{ledgerCode} == 0)
  {
    $error = 1;
    $errorString .= "ledgerCode = '$self->{ledgerCode}' is invalid!<br>\n";
  }
  if ($self->{eventDate} !~ /^\d{2}(-|\/)\d{2}(-|\/)\d{4}$/)
  {
    $error = 1;
    $errorString .= "eventDate = '$self->{eventDate}' is invalid!<br>\n";
  }
  if ($self->{parentId} < -1)
  {
    $error = 1;
    $errorString .= "parentId = '$self->{parentId}' is invalid!<br>\n";
  }
  if ($self->{eventRef} < 0)
  {
    $error = 1;
    $errorString .= "eventRef = '$self->{eventRef}' is invalid!<br>\n";
  }
  if ($self->{journalRef} < -1) # some lot nodes do not need journal entries associated with them
  {
    $error = 1;
    $errorString .= "journalRef = '$self->{journalRef}' is invalid!<br>\n";
  }
  if ($self->{accountNum} < 0)
  {
    $error = 1;
    $errorString .= "accountNum = '$self->{accountNum}' is invalid!<br>\n";
  }
  if ($self->{subAccountNum} < 0)
  {
    $error = 1;
    $errorString .= "subAccountNum = '$self->{subAccountNum}' is invalid!<br>\n";
  }
  if ($self->{lot} < 0)
  {
    $error = 1;
    $errorString .= "lot = '$self->{lot}' is invalid!<br>\n";
  }
  if ($self->{buyLot} < 0)
  {
    $error = 1;
    $errorString .= "buyLot = '$self->{buyLot}' is invalid!<br>\n";
  }
  if ($self->{shares} !~ /^\d+(\.\d{1,6})?$/)
  {
    $error = 1;
    $errorString .= "shares = '$self->{shares}' is invalid!<br>\n";
  }
  if ($self->{cost} !~ /^\d+\.\d{2}$/)
  {
    $error = 1;
    $errorString .= "cost = '$self->{cost}' is invalid!<br>\n";
  }
  if ($self->{nodeId} < 0 || $self->{nodeId} eq "")
  {
    $error = 1;
    $errorString .= "nodeId = '$self->{nodeId}' is invalid!<br>\n";
  }

  $self->{error} = $error;
  $self->{errorString} = "LotNodeObject: isValid() - Error!<br>\n" if $error;
  $self->{errorString} .= $errorString;

  return !$error;
}

sub DESTROY
{
  my $self = shift;
}

sub AUTOLOAD
{
  my $self = shift;
  my $type = ref($self) || die "$self is not an object";
  my $name = $AUTOLOAD;
  $name =~ s/.*://;	# strip fully-qualified portion
  unless (exists $self->{$name})
  {
    die "Can't access `$name' field in object of class $type";
  }
  if (@_)
  {
    return $self->{$name} = shift;
  }
  else
  {
    return $self->{$name};
  }
}

# setError
# requires: errorString
# returns: nothing
sub setError
{
  my $self = shift;
  my %args = ( errorString => "", @_ );
  my $errorString = $args{errorString};
  
  $self->{errorString} = $errorString;
  $self->{error} = 1;
}

# didErrorOccur
# returns the value of error.
sub didErrorOccur
{
  my $self = shift;

  return $self->{error};
}

# errorMessage
# returns the value of errorString
sub errorMessage
{
  my $self = shift;

  return $self->{errorString};
}

1;
__END__

=head1 NAME

LotNodeObject - Object used to build a LotNode Object Class.

=head1 SYNOPSIS

  use Portal::Accounting::Objects::LotNodeObject;
  my $obj = Portal::Accounting::Objects::LotNodeObject->new;
  if ($obj->didErrorOccur())
  {
    die $obj->errorMessage();
  }

=head1 DESCRIPTION

LotNodeObject is a LotNode class.

=head1 Exported FUNCTIONS

  scalar new(ledgerCode, eventDate, parentId, eventRef, journalRef,
             accountNum, subAccountNum, lot, buyLot, shares, cost, nodeId)
    Creates a new instance of the
    Portal::Accounting::Objects::LotNodeObject object.

  bool isValid(void)
    Returns 0 or 1 to indicate if the object is valid.  The error will
    be available via errorMessage().

  void setError(errorString)
    Sets error = 1 and $self->{errorString} = errorString.

  scalar didErrorOccur(void)
    Returns the value of error.

  scalar errorMessage(void)
    Returns the value of errorString.


  NOTE:  All data fields are accessible by specifying the object
         and pointing to the data member to be modified on the
         left-hand side of the assignment.
         Ex.  $obj->variable($newValue); or $value = $obj->variable;

=head1 AUTHOR

PC & Web Xperience, Inc. (mailto:admin@pcxperience.com)

=head1 SEE ALSO

perl(1), Portal(3), Portal::Accounting(3)

=cut
