# PortfolioLotObject.pm - The Object Class that provides a PortfolioLot Object
# Created by James Pattie & Jason A. Pattie, 11/07/2000.
#$Id: PortfolioLotObject.pm,v 1.2 2002/07/11 20:47:01 pcxuser Exp $
#This package is released under the GNU General Public License.
#You should have recieved a copy of the GPL with it.
# Copyright (c) 2000 HLR http://www.pcxperience.org  All rights reserved.

package Portal::Accounting::Objects::PortfolioLotObject;
use strict;
use vars qw($AUTOLOAD $VERSION @ISA @EXPORT);

require Exporter;

@ISA = qw(Exporter AutoLoader);
@EXPORT = qw();
	
$VERSION = '0.01';

# new 
# instantiates an instance of a PortfolioLotObject
# takes: accountNum, subAccountNum, lotRefNum, date, shares, cost, newPricePerShare, marketValue, gainLoss, costPerShare
sub new
{
  my $that = shift;
  my $class = ref($that) || $that;
  my $self = bless {}, $class;
  my %args = ( accountNum => 0, subAccountNum => 0, lotRefNum => 0, date => "01/01/2000", shares => 0, cost => "0.00", newPricePerShare => "0.00", 
               marketValue => "0.00", gainLoss => "0.00", costPerShare => "0.00", buyDate => "01/01/2000", @_ );

  $self->{accountNum} = $args{accountNum};
  $self->{subAccountNum} = $args{subAccountNum};
  $self->{lotRefNum} = $args{lotRefNum};
  $self->{date} = $args{date};
  $self->{shares} = $args{shares};
  $self->{cost} = $args{cost};
  $self->{newPricePerShare} = $args{newPricePerShare};
  $self->{marketValue} = $args{marketValue};
  $self->{gainLoss} = $args{gainLoss};
  $self->{costPerShare} = $args{costPerShare};
  $self->{buyDate} = $args{buyDate};

  # do validation
  $self->{error} = !$self->isValid;  
  if ($self->{error})
  {
    $self->{errorString} = "PortfolioLotObject: new() - Error!<br>\n" . $self->{errorString};
  }

  return $self;
}

# isValid - Returns 0 or 1 to indicate if the object is valid.
sub isValid
{
  my $self = shift;
  my $error = 0;
  my $errorString = "";

  # do validation code here.
  
  if ($self->{accountNum} < 0)
  {
    $error = 1;
    $errorString .= "accountNum = '$self->{accountNum}' is invalid!<br>\n";
  }
  if ($self->{subAccountNum} < 0)
  {
    $error = 1;
    $errorString .= "subAccountNum = '$self->{subAccountNum}' is invalid!<br>\n";
  }
  if ($self->{lotRefNum} < 0)
  {
    $error = 1;
    $errorString .= "lotRefNum = '$self->{lotRefNum}' is invalid!<br>\n";
  }
  if ($self->{date} !~ /^\d{2}(-|\/)\d{2}(-|\/)\d{4}$/)
  {
    $error = 1;
    $errorString .= "date = '$self->{date}' is invalid!<br>\n";
  }
  if ($self->{shares} < 0)
  {
    $error = 1;
    $errorString .= "shares = '$self->{shares}' is invalid!<br>\n";
  }
  if ($self->{cost} !~ /^-?\d+\.\d+$/)
  {
    $error = 1;
    $errorString .= "cost = '$self->{cost}' is invalid!<br>\n";
  }
  if ($self->{newPricePerShare} !~ /^-?\d+\.\d+$/)
  {
    $error = 1;
    $errorString .= "newPricePerShare = '$self->{newPricePerShare}' is invalid!<br>\n";
  }
  if ($self->{marketValue} !~ /^-?\d+\.\d+$/)
  {
    $error = 1;
    $errorString .= "marketValue = '$self->{marketValue}' is invalid!<br>\n";
  }
  if ($self->{gainLoss} !~ /^-?\d+\.\d+$/)
  {
    $error = 1;
    $errorString .= "gainLoss = '$self->{gainLoss}' is invalid!<br>\n";
  }
  if ($self->{costPerShare} !~ /^-?\d+\.\d+$/)
  {
    $error = 1;
    $errorString .= "costPerShare = '$self->{costPerShare}' is invalid!<br>\n";
  }
  if ($self->{buyDate} !~ /^\d{2}(-|\/)\d{2}(-|\/)\d{4}$/)
  {
    $error = 1;
    $errorString .= "buyDate = '$self->{buyDate}' is invalid!<br>\n";
  }

  $self->{error} = $error;
  $self->{errorString} = "PortfolioLotObject: isValid() - Error!<br>\n" if $error;
  $self->{errorString} .= $errorString;

  return !$error;
}

sub DESTROY
{
  my $self = shift;
}

sub AUTOLOAD
{
  my $self = shift;
  my $type = ref($self) || die "$self is not an object";
  my $name = $AUTOLOAD;
  $name =~ s/.*://;	# strip fully-qualified portion
  unless (exists $self->{$name})
  {
    die "Can't access `$name' field in object of class $type";
  }
  if (@_)
  {
    return $self->{$name} = shift;
  }
  else
  {
    return $self->{$name};
  }
}

# setError
# requires: errorString
# returns: nothing
sub setError
{
  my $self = shift;
  my %args = ( errorString => "", @_ );
  my $errorString = $args{errorString};

  $self->{errorString} = $errorString;
  $self->{error} = 1;
}

# didErrorOccur
# returns the value of error.
sub didErrorOccur
{
  my $self = shift;

  return $self->{error};
}

# errorMessage
# returns the value of errorString
sub errorMessage
{
  my $self = shift;
  
  return $self->{errorString};
}

1;
__END__

=head1 NAME

PortfolioLotObject - Object used to build a PortfolioLot Object Class.

=head1 SYNOPSIS

  use Portal::Accounting::Objects::PortfolioLotObject;
  my $portLotObj = Portal::Accounting::Objects::PortfolioLotObject->new(
            account => 1234, subAccount => 2345, lotRefNum => 10,
            date => "02/27/2000", shares => 100, cost => "100.00",
            newPricePerShare => "15.00", marketValue => "1500.00",
            gainLoss => "1400.00", costPerShare => "1.00",
            buyDate => "02/01/2000");
  if ($portLotObj->didErrorOccur())
  {
    die $portLotObj->errorMessage();
  }

=head1 DESCRIPTION

PortfolioLotObject is a PortfolioLot class.  It represents a Lot in the
Portfolio View system for a specific Account/Stock pair.

=head1 Exported FUNCTIONS

  scalar new(account, subAccount, lotRefNum, date, shares, cost,
             newPricePerShare, marketValue, gainLoss, costPerShare, 
             buyDate)
    Creates a new instance of the
    Portal::Accounting::Objects::PortfolioLotObject object.
    newPricePerShare is what the current cost on the market is or what
      we retrieved from historical data for the specified date.
    marketValue = newPricePerShare * shares.
    costPerShare = cost / shares.

  bool isValid(void)
    Returns 0 or 1 to indicate if the object is valid.  The error will
    be available via errorMessage().

  void setError(errorString)
    Sets error = 1 and $self->{errorString} = errorString.

  scalar didErrorOccur(void)
    Returns the value of error.

  scalar errorMessage(void)
    Returns the value of errorString.


  NOTE:  All data fields are accessible by specifying the object
         and pointing to the data member to be modified on the
         left-hand side of the assignment.
         Ex.  $obj->variable($newValue); or $value = $obj->variable;

=head1 AUTHOR

PC & Web Xperience, Inc. (mailto:admin@pcxperience.com)

=head1 SEE ALSO

perl(1)

=cut
