# SubEntityObject.pm - The Object Class that provides a SubEntity Object
#$Id: SubEntityObject.pm,v 1.2 2003/07/29 19:12:49 pcxuser Exp $
#This package is released under the GNU General Public License.
#You should have recieved a copy of the GPL with it.
# Copyright (c) 2000 HLR http://www.pcxperience.org  All rights reserved.

package Portal::Accounting::Objects::SubEntityObject;
use strict;
use Portal::Accounting::Objects::Base;
use vars qw($AUTOLOAD $VERSION @ISA @EXPORT);

require Exporter;

@ISA = qw(Portal::Accounting::Objects::Base Exporter AutoLoader);
@EXPORT = qw();

$VERSION = '0.02';

=head1 NAME

SubEntityObject - Object used to build a SubEntity Object Class.

=head1 SYNOPSIS

  use Portal::Accounting::Objects::SubEntityObject;
  my $obj = Portal::Accounting::Objects::SubEntityObject->new;
  if ($obj->didErrorOccur())
  {
    die $obj->errorMessage();
  }

=head1 DESCRIPTION

SubEntityObject is a SubEntity class.

=head1 Exported FUNCTIONS

B<NOTE>: I<bool> = 1(true), 0(false)

=over 4

=item scalar new(name, code, ledgerCode)

 Creates a new instance of the Portal::Accounting::Objects::SubEntityObject module.
 See Portal::Accounting::Objects::Base(3) for a listing of required arguments.

 requires:
  name
  code
  ledgerCode

=cut

sub new
{
  my $class = shift;
  my $self = $class->SUPER::new(@_);
  my %args = ( name => "", code => "", ledgerCode =>"", @_ );

  if ($self->didErrorOccur)
  {
    $self->prefixError();
    return $self;
  }

  # instantiate anything unique to this module
  $self->{name} = $args{name};
  $self->{number} = $args{code};
  $self->{code} = $args{code};
  $self->{ledgerCode} = $args{ledgerCode};

  # do validation
  if ($self->didErrorOccur)
  {
    $self->prefixError();
    return $self;
  }

  # instantiate anything unique to this module

  return $self;
}

=item bool isValid(void)

 Returns 0 or 1 to indicate if the object is valid.
 The error will be available via errorMessage().

=cut

sub isValid
{
  my $self = shift;

  # make sure our Parent class is valid.
  if (!$self->SUPER::isValid())
  {
    $self->prefixError();
    return 0;
  }

  # validate our parameters.
  if (length $self->{name} == 0)
  {
    $self->invalid("name", $self->{name});
  }
  if (length $self->{ledgerCode} == 0)
  {
    $self->invalid("ledgerCode", $self->{ledgerCode});
  }
  if (length $self->{number} == 0 && length $self->{code} == 0)
  {
    $self->invalid("code", $self->{code});
  }

  if ($self->numInvalid() > 0 || $self->numMissing() > 0)
  {
    $self->postfixError($self->genErrorString("all"));
    return 0;
  }

  return 1;
}

=item string printXML(fullXML bool, indent int)

 This routine returns an XML snippet for this object for use in such
 things as exporting data

 fullXML - defaults to 0 (false).  If true, we output the xml header tag.
 indent - defaults to 2.  Specifies how many spaces in our top level tag
          needs to be.  If fullXML is true, then we force 0, otherwise we
          use what the user specifies.

=cut

sub printXML
{
  my $self = shift;
  my $result = "";
  my %args = ( fullXML => 0, indent => 2, @_ );
  my $fullXML = $args{fullXML};
  my $indent = ($fullXML ? 0 : $args{indent});  # start at 0 indentation if we are making a full XML document.
  my $spaceStr = " " x $indent;

  if ($self->isValid())
  {
    if ($fullXML)
    {
      $result .= "<?xml version=\"1.0\" encoding=\"ISO-8859-1\"?>";
    }
    $result .= $spaceStr . "<subEntity>\n";
    my @entries = qw {name code};
    if ($fullXML)
    {
      push @entries, "ledgerCode";
    }
    foreach my $name (@entries)
    {
      $result .= $spaceStr . "  <$name>$self->{$name}</$name>\n";
    }
    $result .= $spaceStr . "</subEntity>\n";
  }
  else
  {
    $result .= "Data not valid!\n\n";
    $result .= $self->errorMessage();
    die $result;
  }

  return $result;
}

=back

=cut

1;
__END__

=head1 NOTE

 All data fields are accessible by specifying the object
 and pointing to the data member to be modified on the
 left-hand side of the assignment.
 Ex.  $obj->variable($newValue); or $value = $obj->variable;

=head1 AUTHOR

Xperience, Inc. (mailto:admin@pcxperience.com)

=head1 SEE ALSO

perl(1), Portal(3), Portal::Accounting(3), Portal::Accounting::Objects::Base(3)

=cut
