# DateSelect.pm - The Object Class that provides a DateSelect Object
# $Id: DateSelect.pm,v 1.2 2001/09/20 21:30:10 moreejt Exp $
#This package is released under the GPL.  You should have recieved the GPL with it.
# Copyright (c) 2001 PC & Web Xperience, Inc. http://www.pcxperience.com/
# All rights reserved.
=head1 NAME

DateSelect - Object used to build a DateSelect Object Class.

=head1 SYNOPSIS

  use Widgets::DateSelect;
  my $obj = Widgets::DateSelect->new;
  if ($obj->didErrorOccur())
  {
    die $obj->errorMessage();
  }
  $obj->display();

=head1 DESCRIPTION

DateSelect returns a string which is an item to be placed in
an HTML form.  The item is meant to be a date field.  The
style and other options can be customized based on parameters
given to the display command.

=cut

package Portal::Accounting::Widgets::DateSelect;
use strict;
use vars qw($AUTOLOAD $VERSION @ISA @EXPORT);

require Exporter;

@ISA = qw(Exporter AutoLoader);
@EXPORT = qw();

$VERSION = '0.01';

# global variables.

# new
# instantiates an instance of a DateSelect
# optional: browserType, browserVersion, browserMaker
sub new
{
  my $that = shift;
  my $class = ref($that) || $that;
  my $self = bless {}, $class;
  my %args = (browserType => "HTML", browserVersion => "4.0", browserMaker => "", @_ );
  my $errStr = "Widgets::DateSelect->new() - Error!<br>\n";

  $self->{browserType} = $args{browserType};
  $self->{browserVersion} = $args{browserVersion};
  $self->{browserMaker} = $args{browserMaker};

  # do validation
  $self->{error} = !$self->isValid;
  if ($self->{error})
  {
    $self->{errorString} = $errStr . $self->{errorString};
  }

  return $self;
}

# isValid - Returns 0 or 1 to indicate if the object is valid.
sub isValid
{
  my $self = shift;
  my $error = 0;
  my $errorString = "";
  my $errStr = "Widgets::DateSelect->isValid() - Error!<br>\n";

  if ($self->{browserType} !~ /^(HTML|HDML|WML)$/)
  {
    $error = 1;
    $errorString .= "browserType = '$self->{browserType}' is invalid!<br>\n";
  }
  if (length $self->{browserVersion} == 0)
  {
    $error = 1;
    $errorString .= "browserType = '$self->{browserType}' is invalid!<br>\n";
  }

  $self->{error} = $error;
  $self->{errorString} = $errStr if $error;
  $self->{errorString} .= $errorString;

  return !$error;
}

sub DESTROY
{
  my $self = shift;
}

sub AUTOLOAD
{
  my $self = shift;
  my $type = ref($self) || die "$self is not an object";
  my $name = $AUTOLOAD;
  $name =~ s/.*://;	# strip fully-qualified portion
  unless (exists $self->{$name})
  {
    die "Can't access `$name' field in object of class $type";
  }
  if (@_)
  {
    return $self->{$name} = shift;
  }
  else
  {
    return $self->{$name};
  }
}

# setError
# requires: errorString
# returns: nothing
sub setError
{
  my $self = shift;
  my %args = ( errorString => "", @_ );
  my $errorString = $args{errorString};

  $self->{errorString} = $errorString;
  $self->{error} = 1;
}

# didErrorOccur
# returns the value of error.
sub didErrorOccur
{
  my $self = shift;

  return $self->{error};
}

# errorMessage
# returns the value of errorString
sub errorMessage
{
  my $self = shift;

  return $self->{errorString};
}

# displayJavascript - returns string of all needed javascript functions

=head1 string display (... )
Required

Optional
 name: the name to give to the form item, defaults to 'date'
 formName: name of the form that this item is being put into
 month =>, day =>, year => for default date, will override date =>
 date =>:        must conform to the datestyle and separator also sent
 dateStyle:   ymd, dmy, mdy; defaults to ymd
 separator:   defaults to '/'
 java:        default-true; put javascript in place
 size:        defaults to 10
 displayStyle:  1 = 1 edit field, 2 = combo, 3 = 3 edit fields
 table:       true, false to encapsulate the elements within a table
 yLabel:      defaults to nothing, will be used to label the year entry
 mLabel:
 dLabel:
 lables:      top, left, bottom, right
 calendar:    a url pointing to the calendar page
 calendarIcon: a url pointing to the calendar icon
=cut
sub display
{
  my $self;
  $self = shift;
  my %args;
  %args = ( displayStyle => '1',
    calendar => '',
    calendarIcon => '',
    date => '',
    dateStyle => 'ymd',
    name => 'date',
    formName => '',
    separator => '/',
    size => '10',
    java => 'false',
    table => 'true',
    year => '',
    day => '',
    month => '',
    yLabel => '',
    mLabel => '',
    dLabel => '',
    ySize => '4',
    mSize => '2',
    dSize => '2',
    @_ );
  my $calendar  = $args{calendar};
  my $calendarIcon  = $args{calendarIcon};
  my $formName  = $args{formName};
  my $name      = $args{name};
  my $java      = "0";
  my $output;
  my $separator = $args{separator};
  my $size      = $args{size};
  my $displayStyle = $args{displayStyle};
  my $table     = "1";
  my $ySize     = $args{ySize};
  my $mSize     = $args{mSize};
  my $dSize     = $args{dSize};
  my $yLabel    = $args{yLabel};
  my $mLabel    = $args{mLabel};
  my $dLabel    = $args{dLabel};

  my $dateStyle = $args{dateStyle};
  my $y = $args{year};
  my $m = $args{month};
  my $d = $args{day};
  my $date = $args{date};

  my $c1; #dateStyle holders
  my $c2;
  my $c3;

  if ($dateStyle !~ /^(ymd|ydm|myd|mdy|dmy|dmy)$/)
  {
    $self->setError(errorString => "Invalid dateStyle: $dateStyle\n");
    return undef;
  }
  ($c1, $c2, $c3) = split("",$dateStyle);
  if ($y eq "" && $d eq "" && $m eq "")
  {
    if ($date ne "")
    {
      my @array = split($separator,$date);
      if (scalar @array != 3)
      {
        $self->setError(errorString => "Invalid date or separator D:'$date' S:'$separator'\n");
        return undef;
      }

      eval("\$" . $c1 ." = \"" . $array[0] . "\";");
      eval("\$" . $c2 ." = \"" . $array[1] . "\";");
      eval("\$" . $c3 ." = \"" . $array[2] . "\";");

      if ($y !~ /^(\d+)(\d*)$/ && $m !~ /^[0-12]$/ && $d !~ /^[1-31]$/)
      {
        $self->setError(errorString => "Invalid date DateSt: '$dateStyle' '$date' ~ Y:'$y' M:'$m' D:'$d' c1:$c1 c2:$c2 c3:$c3\n");
        return undef;
      }
    }
  }
  elsif ($y =~ /^(\d)+$/ && $m =~ /^[0-12]$/ && $d =~ /^[1-31]$/)
  { #dereference c1,2,3 with y,m,d and use those as variables
    $date = ${$c1} . $separator . ${$c2} . $separator . ${$c3};
  }
  else
  {
    $self->setError(errorString => "Invalid date:'$date' Y:'$y' M:'$m' D:'$d'\n");
    return undef;
  }

  if ($displayStyle !~ /^(1|2|3)$/)
  {
    $self->setError(errorString => "Invalid displayStyle: $displayStyle\n");
    return undef;
  }
  if ($calendar !~ /^http/)
  {
    if ($calendar ne "")
    {
      $self->setError(errorString => "Invalid calendar url: $calendar\n");
      return undef;
    }
  }
  else
  {
    if ($calendarIcon eq "")
    {
      $calendarIcon = "&lt;&gt;";
    }
    elsif ($calendarIcon !~ /^http/)
    {
      $self->setError(errorString => "Invalid calendar icon url: $calendarIcon\n");
      return undef;
    }
    $calendarIcon = "<img=\"" . $calendarIcon . "\" border=\"0\">";
  }
  if ($args{table} eq "false")
  {
    $table = 0;
  }
  if ($args{java} eq "true")
  {
    $java = 1;
  }

  if ($table)
  {
    $output .= "<table>";
    $output .= "<tr><td align=\"center\">\n";
  }
  if ($displayStyle eq "2" || $displayStyle eq "3")
  { #will add label location later!!!
    eval("\$output .= \$".$c1."Label . \"<input size=\\\"\$".$c1.
      "Size\\\" type=\\\"text\\\" name=\\\"$c1"."_$name\\\" value=\\\"\$"
      . $c1."\\\">\" ; ");
#    $output .= ${$c1."Label"}."<input size=".${$c1."Size"}." type=\"text\" name=\""
#      . $c1."_".$name."\" value=\"". ${$c1}."\">";
    eval( "\$output .= \$".$c2."Label . \"<input size=\\\"\$".$c2.
      "Size\\\" type=\\\"text\\\" name=\\\"$c2"."_$name\\\" value=\\\"\$"
      . $c2."\\\">\" ; ");
#    $output .= ${$c2."Label"}."<input size=".${$c2."Size"}." type=\"text\" name=\""
#      . $c2."_".$name."\" value=\"". ${$c2}."\">";
    eval("\$output .= \$".$c3."Label . \"<input size=\\\"\$".$c3.
      "Size\\\" type=\\\"text\\\" name=\\\"$c3"."_$name\\\" value=\\\"\$"
      . $c3."\\\">\" ; ");
#    $output .= ${$c3."Label"}."<input size=".${$c3."Size"}." type=\"text\" name=\""
#      . $c3."_".$name."\" value=\"". ${$c3}."\">";
  }
  if ($displayStyle eq "2" || $displayStyle eq "1")
  {
    if ($displayStyle eq "2")
    {
      $output .= "<br>\n";
    }

    $output .= "<input type=\"text\" size=\"$size\" name=\"$name\" value=\"$date\">\n";
    if ($java && $calendar ne "")
    {
      $output .= "<a href=\"#\"";
      $output .= " onClick=\"window.dateField = document.$formName.$name;DateSelect = window.open('"
          . $calendar .
          "', 'cal', 'WIDTH=200,HEIGHT=400')\" onmouseover=\"window.status='PopUp DateSelect'; return true;\">";
      $output .= $calendarIcon;
      $output .=  "</a>\n";
  #       $output .= " onClick=\"window.dateField = document.$formName.$name;DateSelect = window.open('/widgets/DateSelect.html', 'cal', 'WIDTH=200,HEIGHT=400')\" onmouseover=\"window.status='PopUp DateSelect'; return true;\"><img src=\"/widgets/DateSelect.gif\" border=\"0\" alt=\"PopUp DateSelect\">";
    }
  }

  if ($table)
  {
    $output .= "</td></tr></table>\n";
  }
  return $output;
}

1;
__END__


=head1 Exported FUNCTIONS

  scalar new(browserType, browserVersion, browserMaker)
    Creates a new instance of the Widgets::DateSelect
    object.

  bool isValid(void)
    Returns 0 or 1 to indicate if the object is valid.  The error will
    be available via errorMessage().

  void setError(errorString)
    Sets error = 1 and errorString = errorString.

  scalar didErrorOccur(void)
    Returns the value of error.

  scalar errorMessage(void)
    Returns the value of errorString.


  NOTE:  All data fields are accessible by specifying the object
         and pointing to the data member to be modified on the
         left-hand side of the assignment.
         Ex.  $obj->variable($newValue); or $value = $obj->variable;

=head1 AUTHOR

JT Moree
PC & Web Xperience, Inc. (mailto:admin@pcxperience.com)

=head1 SEE ALSO

perl(1), Widgets(3)

=cut
